@@ -1,6 +1,6 @@
 #!/usr/bin/perl -T
 ###
-###              analog 4.16             http://www.analog.cx/
+###              analog 5.01             http://www.analog.cx/
 ### This program is copyright (c) Stephen R. E. Turner 1995 - 2001 except as
 ### stated otherwise. Distribution, usage and modification of this program is
 ### subject to the conditions of the Licence which you should have received
@@ -13,8 +13,8 @@
 
 # 1) uncomment (remove everything before $analog) and edit one of the next two
 #    lines to give the location (full pathname) of the analog executable.
-#       Unix: $analog = '/usr/local/etc/httpd/analog-4.16/analog';
-#       Windows: $analog = 'C:\program files\analog 4.16\analog.exe';
+#       Unix: $analog = '/usr/local/etc/httpd/analog-5.01/analog';
+#       Windows: $analog = 'C:\program files\analog 5.01\analog.exe';
 # 2) If you're on Unix, edit the first line in this file to give the location
 #    of Perl (don't remove the #! though).
 # 3) You also need to edit anlgform.html if you want to use the form.
@@ -22,7 +22,8 @@
 
 @forbidden = qw(LOGFORMAT APACHELOGFORMAT DEFAULTLOGFORMAT
 		APACHEDEFAULTLOGFORMAT HEADERFILE FOOTERFILE UNCOMPRESS
-		OUTFILE CACHEOUTFILE ERRFILE DNS CGI SETTINGS LANGFILE);
+		OUTFILE CACHEOUTFILE LOCALCHARTDIR ERRFILE DNS CGI
+		SETTINGS LANGFILE DESCFILE);
 # Forbidden commands: sysadmin can add more (must be in upper case!)
 # Other commands you might consider adding, because they allow users to
 # specify which files to use for the analysis, are LOGFILE and DOMAINSFILE.
@@ -33,7 +34,9 @@ require 5.001;
 use CGI;
 
 # 1) INITIALISATION
-$ENV{PATH} = '';
+# delete all dangerous environment variables
+$ENV{PATH} = '';  # blank not deleted so that UNCOMPRESS doesn't get a path
+delete @ENV{qw/IFS CDPATH ENV BASH_ENV/};
 $query = new CGI;
 $|=1;
 $lt = localtime;
@@ -146,7 +149,7 @@ sub printargs {
 # B) PUT APPROPRIATE DELIMITERS ROUND AN ARGUMENT CONTAINING SPACES
 sub bracket {
     local $_ = $_[0];
-    return $_ unless (/[\s\#]/ || /^['"\(]/);
+    return $_ unless (/[\s\#]/ || /^['"\(]/ || /\\$/);
     return "\"$_\"" unless (/"/);
     return "'$_'" unless (/'/);
     return "($_)";
@@ -157,7 +160,9 @@ sub bracket {
 sub checkfilechars {
     local ($_);
     foreach (@{$args{$_[0]}}, @{$args{$_[0] . '1'}}) {
-	if (m([^\w\. /\\:-]) || m(\W-|^-|-\W|-$)) { # NB \w includes underscore
+	if (m([^\w. /\\:\-*?]) || m(\B-|-\B)) {
+	    # i.e. contains a non-approved character, or a dash not
+	    # between \w's. NB \w includes underscore.
 	    badreq(403, "Illegal Request", "Unsafe characters in $_[0].");
 	    printf STDERR "[$lt] $progname: Unsafe characters in \"$_[0] $_\" on request from %s\n", $ENV{REMOTE_HOST}?$ENV{REMOTE_HOST}:($ENV{REMOTE_ADDR}?$ENV{REMOTE_ADDR}:"unknown host");
 	    die;
@@ -165,7 +170,7 @@ sub checkfilechars {
     }
 }
 
-# D) CHECK ONLY SAFE CHARACTERS IN OTHER COMMANDS. Again, see docs/form.html.
+# D) CHECK NO UNSAFE CHARACTERS IN OTHER COMMANDS. Again, see docs/form.html.
 sub checkchars {
     local $_ = $_[0];
     if (/[\x00-\x1F\x7F-\x9F]/) {
